"""ApiPosturePro - Professional security analysis for Python REST APIs."""

__version__ = "1.0.0"
__author__ = "ApiPosture Team"

from apiposture_pro.licensing.manager import ProLicenseManager
from apiposture_pro.licensing.models import LicenseContext, StoredLicense

__all__ = [
    "__version__",
    "ProLicenseManager",
    "LicenseContext",
    "StoredLicense",
]
